#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <AVKit/AVKit.h>
#import <SmartclipSDKiOS/SCAdFacadeDelegate.h>
#import <SmartclipSDKiOS/SCElementSize.h>
#import <SmartclipSDKiOS/SCAdSessionStateListener.h>

NS_ASSUME_NONNULL_BEGIN

@class SCAdConfiguration, SCAdViewConfiguration, SCAdEvent, SCAdSessionController, SCElementSize;

/// Delegate functions that need to be implemented to use this class correctly
@protocol SCAdControllerDelegate <NSObject>
@required
/// return the size of the player
- (CGRect)playerSize;
/// return the size of the viewport
- (CGRect)viewportSize;
/// called when an alert should be displayed by the view controller
- (void)displayAlert:(UIAlertController*)alert;
/// called when the adSlot has started
- (void)adSlotStarted;
/// called when the adSlot has completed
- (void)adSlotCompleted;
/// called when the skip button should be displayed or hidden
- (void)hideSkipButton:(BOOL)hide;
/// called to set the correct state of the mute button (muted or not muted)
- (void)displayMuted:(BOOL)muted;
/// called with values to customize you ui (text, textcolor and progressbar color)
- (void)configureDisplayWith:(SCAdViewConfiguration*)adViewConfiguration;
/// called when the progressBar should be displayed or hidden
- (void)displayProgress:(BOOL)display;
/// called with the current progress value to be displayed by the progressBar
- (void)updateProgress:(CGFloat)progress;
/// give back the current view mode (ViewModeNormal or ViewModeFullscreen)
- (ViewMode)currentViewMode;
/// called when allover playback has finished
- (void)playbackFinished;

@optional
/// called to signal if a fullscreen mode should be exited after playback of the current ad
- (void)exitFullscreenAfterPlayback:(BOOL)exit;

@optional
/// called for every happening SCAdEvent, can be used to add customized code
/// for certain events
- (void)onEventCallbackWith:(SCAdEvent*)event;

@optional
/// if you have set customizableClickThroughAlert to true in the configuration
/// you must implement this callback function and display the clickThroughAlert
/// by yourself when this callback is called
- (void)displayClickThroughAlert;
@end

@protocol SCAdControllerLoggingDelegate <NSObject>
- (void)log:(NSString*)string;
@end

/// CompletionBlock that delivers a double value as result
typedef void (^SCDoubleResultCompletionBlock)(double result);

/**
 Use this class to simplify ad playback for standalone players (or as a base class for
 any other type of adController based playback)
 */
@interface SCSimpleAdController : NSObject <SCAdSessionStateListener, SCAdListener>
/// init with sessionController instance
- (instancetype)initWithSessionController:(SCAdSessionController*)sessionController;
/// set the facade delegate (your PlayerController e.g. AVPlayerController)
- (void)setFacadeDelegate:(id<SCAdFacadeDelegate>)facadeDelegate;
/// set the configuration for the upcoming adSlot
- (void)setConfiguration:(SCAdConfiguration*)configuration;
/// set the player view that holds the (AV)player
- (void)setPlayerView:(UIView*)view;
/// add every view to the friendly views array that covers the adView
- (void)registerFriendlyViews:(NSMutableArray*)views;
/// call resize every time the size of the player changes
- (void)resize;
/// call to toggle mute/unmute
- (void)mute;
/// call to skip the ad (if the skipButton is visible)
- (void)skip;
/// call to open clickThrough alert when user taps on player
- (void)onClickThrough;
/// check if the player is currently muted
- (BOOL)isMuted;
/// get duration of the current clip
- (double)getDuration;
/// call this function to stop the adSlot
- (void)stopAdSlot;
/// call this function to cleanup everything
- (void)cleanup;
/// if an error occured (event ON_AD_ERROR) a more detailed error
/// description can be accessed here
- (void)getAdError:(SCAdErrorCompletionBlock)completion;
/// get the clickThrough url
- (void)clickThroughURL:(SCStringResultCompletionBlock)completion;
/// pause advertisement playback
- (void)pauseAdPlayback;
/// resume advertisement playback
- (void)resumeAdPlayback;

- (void)displayRealProgress;

- (double)setProgressValuesAndGetUpdateInterval:(SCPublicAdSlot*_Nonnull)adSlot;

- (void)stopProgressTimer;

/// set your delegate
@property (nonatomic, weak) id<SCAdControllerDelegate> _Nullable delegate;

/// if you want to log detailed information, set a logging delegate
@property (nonatomic, weak) id<SCAdControllerLoggingDelegate> _Nullable loggingDelegate;

@property (nonatomic) double allOverDuration;
@property (nonatomic) double progress;
@property (nonatomic) double recentClipsTime;

@property (nonatomic, strong) SCAdConfiguration      * _Nullable config;
@property (nonatomic, strong) SCAdSessionController  * _Nullable sessionController;
@property (nonatomic, strong) SCAdSlotController     * _Nullable adSlotController;
@property (nonatomic, weak) id<SCAdFacadeDelegate>   _Nullable avPlayerController;

@property (nonatomic, strong) NSTimer * _Nullable progressTimer;
@end

NS_ASSUME_NONNULL_END
